# -*- coding: utf8 -*-
import logging

import datetime
from efa.lib.report import (
    filter_complete_entry,
    filter_complete_exit,
    filter_items,
    filter_broken_participations,
    init_evaluation_result,
    evaluate_participation,
    get_entry_values,
    get_exit_values,
    get_first,
    get_last,
    merge_values,
    filter_date_range,
)

log = logging.getLogger(__name__)

filters = [('gender', [0, 1])]
indicators = {'OI': [
                ('CO01', u'Arbeitslose, auch Langzeitarbeitslose (CO01)'),
                ('CO02', u'Langzeitarbeitslose (CO02)'),
                ('CO03', u'Nichterwerbstätige (CO03)'),
                ('CO04', u'Nichterwerbstätige, die keine schulische oder berufliche Bildung absolvieren (CO04)'),
                ('CO05', u'Erwerbstätige, auch Selbständige (CO05)'),
                ('CO06', u'Unter 25-Jährige (CO06)'),
                ('CO07', u'Über 54-Jährige (CO07)'),
                ('CO08', u'Über 54-Jährige, die arbeitslos sind, einschließlich Langzeitarbeitsloser, oder die nicht erwerbstätig sind und keine schulische oder berufliche Bildung absolvieren (CO08)'),
                ('CO09', u'Mit Grundbildung (ISCED 1) oder Sekundarbildung Unterstufe (ISCED 2) (CO09)'),
                ('CO10', u'Mit Sekundarbildung Oberstufe (ISCED 3) oder postsekundärer Bildung (ISCED 4) (CO10)'),
                ('CO11', u'Mit tertiärer Bildung (ISCED 5 bis 8) (CO11)'),
                ('CO15', u'Migranten, Teilnehmer ausländischer Herkunft, Angehörige von Minderheiten (u.a. marginalisierte Gemeinschaften, wie etwa die Roma) (CO15)'),
                ('CO16', u'Teilnehmer mit Behinderungen (CO16)'),
                ('CO17', u'Sonstige benachteiligte Personen (CO17)'),
                ('CO18', u'Obdachlose oder von Ausgrenzung auf dem Wohnungsmarkt Betroffene (CO18)')
            ],
            'RI': [
                ('CR01', u'Nichterwerbstätige Teilnehmer, die nach ihrer Teilnahme auf Arbeitsuche sind (CR01)'),
                ('CR02', u'Teilnehmer, die nach ihrer Teilnahme eine schulische/berufliche Bildung absolvieren (CR02)'),
                ('CR03', u'Teilnehmer, die nach ihrer Teilnahme eine Qualifizierung erlangen (CR03)'),
                ('CR04', u'Teilnehmer, die nach ihrer Teilnahme einen Arbeitsplatz haben, einschließlich Selbständige (CR04)'),
                ('CR05', u'Benachteiligte Teilnehmer, die nach ihrer Teilnahme auf Arbeitsuche sind, eine schulische/berufliche Bildung absolvieren, eine Qualifizierung erlangen, einen Arbeitsplatz haben, einschließlich Selbständige (CR05)')
            ]
    }


def recipe(participations, validate, program):

    def _has_valid_entries(values):
        valid = validate(values)
        return not valid.has_violated_rules()

    # helper for handling fp1
    def _check_on_fp1(personal_id):
        part1 = personal_id.split("-")[1]
        if part1.startswith("0"):
            return True
        if part1.startswith("1"):
            return False
        if part1[1] == "0":
            return True
        if part1[1] == "1":
            return False
        raise ValueError("Cannot map personal_id {} to fp1 or fp2".format(
            personal_id))

    def _check_start_date_in_fp1(participation):
        return participation.start_date <= datetime.date(2018, 12, 31)

    def _check_end_date_in_fp1(participation):
        if not participation.end_date:
            return True
        return participation.end_date <= datetime.date(2018, 12, 31)

    # Handle entry data.
    complete_entry = filter_complete_entry(participations)
    # fp1 special case
    if complete_entry:
        if _check_on_fp1(complete_entry[0].participant.personal_id):
            complete_entry = filter_items(complete_entry,
                                          _check_start_date_in_fp1)

    values_entry = get_entry_values(complete_entry, program+"oi")
    # Filter out values with violated ESF Rules. This is a extra
    # requirement for the "Durchführungsbericht"
    values_valid_entry = filter_items(values_entry, _has_valid_entries)
    first_entry = get_first(values_valid_entry)

    if first_entry is None:
        return {}

    # Handle exit data
    complete_exit = filter_complete_exit(participations)
    # fp1 special case
    if complete_exit:
        if _check_on_fp1(complete_exit[0].participant.personal_id):
            complete_exit = filter_items(complete_exit,
                                         _check_end_date_in_fp1)
    # Get the minimal set of values from the filtered
    # participations which are needed for the later evaluation.
    values_exit = get_exit_values(complete_exit, program+"ri")
    exit_candidates = filter_date_range(values_exit, "end_date",
                                        first_entry["start_date"],
                                        datetime.date.today())
    last_exit = get_last(exit_candidates)
    merged = merge_values(first_entry, last_exit)

    # We do not except validation errors here. We believe that the rules
    # for the resultindicators can not fail. However we will log a
    # warning if the validation fails.
    if validate(merged).has_violated_rules():
        f_id, l_id = "?", "?"
        if first_entry:
            f_id = first_entry.get("id")
        if last_exit:
            l_id = last_exit.get("id")
        log.warning("ESF Validation in 'Durchführungsbericht' for "
                    "participations (entry, exit) %s, %s failed."
                    % (f_id, l_id))
    return merged


def evaluate(participations,
             start, end, filter_type,
             oi_fields, ri_fields,
             filters, foerderphase=None):
    """A simple report generator"""
    participations = filter_broken_participations(participations)
    if foerderphase == 0:
        participations = [parti for parti in participations
                          if parti.get('start_date') < datetime.date(2019,
                                                                     1, 1)]
    elif foerderphase == 1:
        participations = [parti for parti in participations
                          if parti.get('start_date') > datetime.date(2018,
                                                                     12, 31)]
    # No filter at all (-1)
    if filter_type == -1:
        entries = participations
        exits = entries
    # Start and End within timeframe (1)
    elif filter_type == 1:
        entries = filter_date_range(participations, "start_date", start, end)
        entries = filter_date_range(entries, "end_date", start, end)
        exits = entries
    # Start within timeframe (2)
    elif filter_type == 2:
        entries = filter_date_range(participations, "start_date", start, end)
        exits = filter_date_range(entries, "end_date", start, end)
    # End within timeframe (3)
    elif filter_type == 3:
        exits = filter_date_range(participations, "end_date", start, end)
        entries = filter_date_range(exits, "start_date", start, end)
    # Start or End within timeframe (4)
    elif filter_type == 4:
        entries = filter_date_range(participations, "start_date", start, end)
        exits = filter_date_range(participations, "end_date", start, end)

    entries_m = sum([1 for pn in entries if pn.get("gender") == 1])
    entries_w = sum([1 for pn in entries if pn.get("gender") == 0])
    exits_m = sum([1 for pn in exits if pn.get("gender") == 1])
    exits_w = sum([1 for pn in exits if pn.get("gender") == 0])

    out = init_evaluation_result(oi_fields, filters)
    out2 = init_evaluation_result(ri_fields, filters)

    for participation in entries:
        evaluate_participation(out, participation)

    for participation in exits:
        evaluate_participation(out2, participation)

    out.update(out2)

    entries_total = entries_m + entries_w
    exits_total = exits_m + exits_w
    OIs = [indicator[0] for indicator in indicators["OI"]]
    for indicator in out:
        indicator_total = out[indicator]['gender']['total']
        divisor = entries_total if indicator in OIs else exits_total
        if divisor > 0:
            out[indicator]['gender']['totalpercent'] = \
                str(round(100.0 * float(indicator_total)
                    / float(divisor), 1)).replace(".", ",")
        else:
            out[indicator]['gender']['totalpercent'] = "K.A."
    out['exits'] = (exits_m, exits_w, exits_m + exits_w)
    out['entries'] = (entries_m, entries_w, entries_m + entries_w)
    out['start_date'] = start
    out['end_date'] = end
    out['filter_type'] = filter_type
    out['foerderphase'] = foerderphase
    return out
