# -*- coding: utf8 -*-
import logging
from collections import namedtuple
from datetime import date

from efa.lib.report import (
    filter_complete_entry,
    filter_complete_exit,
    get_entry_values,
    get_exit_values,
    get_first,
    get_last,
    merge_values,
    filter_date_range,
    filter_broken_participations,
    init_evaluation_result,
    evaluate_participation
)
from efa.model.beneficiary import Beneficiary, get_beneficiary_configuration_by_date
from ringo_bmasreport.reports.durchfuehrungsbericht import recipe

log = logging.getLogger(__name__)

indicators = {'OI': [],
              'RI': [
    ('26', u'Der/Die Teilnehmende war innerhalb von vier Wochen nach Austritt aus dem Projekt neu bei einer Agentur für Arbeit bzw. einem Jobcenter arbeitsuchend gemeldet'),
    ('27', u'Der/Die Teilnehmende war innerhalb von vier Wochen nach Austritt aus dem Projekt in einer schulischen/ beruflichen Bildung'),
    ('28', u'Der/Die Teilnehmende hat innerhalb von vier Wochen nach Austritt aus der Maßnahme/ dem Projekt nachweislich eine Qualifizierung erhalten'),
    ('29', u'Der/Die Teilnehmende hat innerhalb von vier Wochen nach Austritt aus dem Projekt eine Arbeit aufgenommen oder hat sich selbstständig gemacht')]}
filters = [('gender', [0, 1])]

Line = namedtuple("Line", "text male female total totalpercent")
Entry = namedtuple("Entries", "male female total")


def evaluate(participations,
             start, end, filter_type,
             oi_fields, ri_fields, filters):
    """A simple report generator"""
    entries = filter_date_range(participations, "start_date", start, end)
    exits = filter_broken_participations(participations)
    exits = filter_date_range(exits, "end_date", start, end)

    entries_m = sum([1 for pn in entries if pn.get("gender") == 1])
    entries_w = sum([1 for pn in entries if pn.get("gender") == 0])
    exits_m = sum([1 for pn in exits if pn.get("gender") == 1])
    exits_w = sum([1 for pn in exits if pn.get("gender") == 0])

    out = init_evaluation_result(oi_fields, filters)
    out2 = init_evaluation_result(ri_fields, filters)

    # Iterate through the participations and update the result list for each
    # combination of BMAS field and field from the filter list
    for participation in exits:
        evaluate_participation(out, participation)
    for participation in exits:
        evaluate_participation(out2, participation)

    out.update(out2)

    # Now calculate the totalpercent = indicator_total / exits_total in Percent
    exits_total = exits_m + exits_w
    for indicator in out:
        indicator_total = out[indicator]['gender']['total']
        if exits_total > 0:
            out[indicator]['gender']['totalpercent'] = \
                str(round(100.0 * float(indicator_total)
                          / float(exits_total), 1)).replace(".", ",")
        else:
            out[indicator]['gender']['totalpercent'] = "K.A."

    out['exits'] = (exits_m, exits_w, exits_m + exits_w)
    out['entries'] = (entries_m, entries_w, entries_m + entries_w)
    out['start_date'] = start
    out['end_date'] = end
    return out


def generate_line(indicator, values):
    return Line(indicator[1],
                values["gender"][1],
                values["gender"][0],
                values["gender"]["total"],
                values["gender"]["totalpercent"])


def get_beneficiary_id_by_start_date(db, start_date):
    bc = get_beneficiary_configuration_by_date(db, start_date)
    return bc.beneficiary_id


def prepare_result_for_print(data, request):
    out = data.get("out")
    indicators = data.get("indicators")
    oi = indicators.get("OI")
    ri = indicators.get("RI")
    ri_text = [generate_line(i, out[i[0]]) for i in ri]
    oi_text = [generate_line(i, out[i[0]]) for i in oi]
    vorhaben = (request.db.query(Beneficiary).all())
    programname = format_programmname(request)
    fnumber = get_beneficiary_id_by_start_date(
        request.db, out.get('start_date'))
    zeitraum = out["start_date"].strftime(
        '%d.%m.%Y')+" bis "+out["end_date"].strftime('%d.%m.%Y')
    result = {
        "indicators": ri_text,
        "oiindicators": oi_text,
        "programname": programname,
        "entries": Entry(out["entries"][0], out["entries"][1], out["entries"][0] + out["entries"][1]),
        "exits": Entry(out["exits"][0], out["exits"][1], out["exits"][0] + out["exits"][1]),
        "vorhaben": ", ".join([v.name+" ("+fnumber+")" for v in vorhaben]),
        "erstellt": date.today().strftime('%d.%m.%Y'),
        "zeitraum": zeitraum
    }
    return result


def format_programmname(request):
    programname = request.registry.settings.get('app.title')
    names = {
        "EC": u"Elternchance II",
        "SIB": u"Stark im Beruf – Mütter mit Migrationshintergrund steigen ein",
        "QE": u"Quereinstieg – Männer und Frauen in Kitas",
        "PWE": u"Perspektive Wiedereinstieg – Potenziale erschließen"
    }
    return names.get(programname)
